#!/usr/bin/env python
import os
import sys
import warnings

import django
from django.core.management import execute_from_command_line

parent = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, parent)

DEFAULT_TEST_APPS = [
    'sortedm2m_tests',
]


def runtests(*args):
    warnings.filterwarnings("ignore", module="distutils")

    # Ignore a python 3.6 DeprecationWarning in ModelBase.__new__ that isn't
    # fixed in Django 1.x
    if sys.version_info > (3, 6) and django.VERSION < (2,):
        warnings.filterwarnings(
            "ignore", "__class__ not set defining", DeprecationWarning)

    test_apps = list(args or DEFAULT_TEST_APPS)
    print([sys.argv[0], 'test', '--verbosity=1'] + test_apps)
    execute_from_command_line([sys.argv[0], 'test', '--verbosity=1'] + test_apps)


if __name__ == '__main__':
    runtests(*sys.argv[1:])



from django.db import models
from django.utils.translation import ugettext_lazy as _

from sortedm2m.fields import SortedManyToManyField


class Photo(models.Model):
    name = models.CharField(max_length=50)

    class Meta:
        verbose_name = _('Photo')
        verbose_name_plural = _('Photos')

    def __str__(self):
        return self.name


class Gallery(models.Model):
    name = models.CharField(max_length=50)
    photos = SortedManyToManyField(Photo)
    photos2 = SortedManyToManyField(Photo, related_name='gallery2+')

    class Meta:
        verbose_name = _('Photo')
        verbose_name_plural = _('Photos')

    def __str__(self):
        return self.name



from functools import partial

from django.db import models, router, transaction
from django.db.models import Max, Model, signals
from django.db.models.fields.related import ManyToManyField as _ManyToManyField
from django.db.models.fields.related import lazy_related_operation, resolve_relation
from django.db.models.fields.related_descriptors import ManyToManyDescriptor, create_forward_many_to_many_manager
from django.db.models.utils import make_model_tuple
from django.utils.encoding import force_str
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _

from .compat import get_rel
from .forms import SortedMultipleChoiceField

SORT_VALUE_FIELD_NAME = 'sort_value'


def create_sorted_many_related_manager(superclass, rel, *args, **kwargs):
    RelatedManager = create_forward_many_to_many_manager(
        superclass, rel, *args, **kwargs)

    class SortedRelatedManager(RelatedManager):
        def _apply_rel_ordering(self, queryset):
            return queryset.extra(order_by=['%s.%s' % (
                self.through._meta.db_table,
                self.through._sort_field_name,  # pylint: disable=protected-access
            )])

        def get_queryset(self):
            # We use ``extra`` method here because we have no other access to
            # the extra sorting field of the intermediary model. The fields
            # are hidden for joins because we set ``auto_created`` on the
            # intermediary's meta options.
            try:
                # pylint: disable=protected-access
                return self.instance._prefetched_objects_cache[self.prefetch_cache_name]
            except (AttributeError, KeyError):
                queryset = super(SortedRelatedManager, self).get_queryset()
                return self._apply_rel_ordering(queryset)

        def get_prefetch_queryset(self, instances, queryset=None):
            # Apply the same ordering for prefetch ones
            result = super(SortedRelatedManager, self).get_prefetch_queryset(instances, queryset)
            return (self._apply_rel_ordering(result[0]),) + result[1:]

        def set(self, objs, **kwargs):  # pylint: disable=arguments-differ
            # Choosing to clear first will ensure the order is maintained.
            kwargs['clear'] = True
            super(SortedRelatedManager, self).set(objs, **kwargs)
        set.alters_data = True

        # pylint: disable=arguments-differ
        def _add_items(self, source_field_name, target_field_name, *objs, **kwargs):
            # source_field_name: the PK fieldname in join table for the source object
            # target_field_name: the PK fieldname in join table for the target object
            # *objs - objects to add. Either object instances, or primary keys of object instances.
            # **kwargs: in Django >= 2.2; contains `through_defaults` key.
            through_defaults = kwargs.get('through_defaults') or {}

            # If there aren't any objects, there is nothing to do.
            if objs:
                # Django uses a set here, we need to use a list to keep the
                # correct ordering.
                new_ids = []
                for obj in objs:
                    if isinstance(obj, self.model):
                        if not router.allow_relation(obj, self.instance):
                            raise ValueError(
                                'Cannot add "%r": instance is on database "%s", value is on database "%s"' %
                                (obj, self.instance._state.db, obj._state.db)  # pylint: disable=protected-access
                            )

                        fk_val = self.through._meta.get_field(target_field_name).get_foreign_related_value(obj)[0]

                        if fk_val is None:
                            raise ValueError(
                                'Cannot add "%r": the value for field "%s" is None' %
                                (obj, target_field_name)
                            )

                        new_ids.append(fk_val)
                    elif isinstance(obj, Model):
                        raise TypeError(
                            "'%s' instance expected, got %r" %
                            (self.model._meta.object_name, obj)
                        )
                    else:
                        new_ids.append(obj)

                db = router.db_for_write(self.through, instance=self.instance)
                manager = self.through._default_manager.using(db)  # pylint: disable=protected-access
                vals = (self.through._default_manager.using(db)  # pylint: disable=protected-access
                        .values_list(target_field_name, flat=True)
                        .filter(**{
                            source_field_name: self.related_val[0],
                            '%s__in' % target_field_name: new_ids,
                        }))

                # make set.difference_update() keeping ordering
                new_ids_set = set(new_ids)
                new_ids_set.difference_update(vals)

                new_ids = list(filter(lambda _id: _id in new_ids_set, new_ids))

                # Add the ones that aren't there already
                with transaction.atomic(using=db, savepoint=False):
                    if self.reverse or source_field_name == self.source_field_name:
                        # Don't send the signal when we are inserting the
                        # duplicate data row for symmetrical reverse entries.
                        signals.m2m_changed.send(
                            sender=self.through, action='pre_add',
                            instance=self.instance, reverse=self.reverse,
                            model=self.model, pk_set=new_ids_set, using=db,
                        )

                    rel_source_fk = self.related_val[0]
                    rel_through = self.through
                    sort_field_name = rel_through._sort_field_name  # pylint: disable=protected-access

                    # Use the max of all indices as start index...
                    # maybe an autoincrement field should do the job more efficiently ?
                    source_queryset = manager.filter(**{'%s_id' % source_field_name: rel_source_fk})
                    sort_value_max = source_queryset.aggregate(max=Max(sort_field_name))['max'] or 0

                    bulk_data = [
                        dict(through_defaults, **{
                            '%s_id' % source_field_name: rel_source_fk,
                            '%s_id' % target_field_name: obj_id,
                            sort_field_name: obj_idx,
                        })
                        for obj_idx, obj_id in enumerate(new_ids, sort_value_max + 1)
                    ]

                    manager.bulk_create([rel_through(**data) for data in bulk_data])

                    if self.reverse or source_field_name == self.source_field_name:
                        # Don't send the signal when we are inserting the
                        # duplicate data row for symmetrical reverse entries.
                        signals.m2m_changed.send(
                            sender=self.through, action='post_add',
                            instance=self.instance, reverse=self.reverse,
                            model=self.model, pk_set=new_ids_set, using=db,
                        )

    return SortedRelatedManager


class SortedManyToManyDescriptor(ManyToManyDescriptor):
    def __init__(self, field):
        super(SortedManyToManyDescriptor, self).__init__(field.remote_field)

    @cached_property
    def related_manager_cls(self):
        model = self.rel.model
        return create_sorted_many_related_manager(
            model._default_manager.__class__,  # pylint: disable=protected-access
            self.rel,
            # This is the new `reverse` argument (which ironically should
            # be False)
            reverse=False,
        )


class SortedManyToManyField(_ManyToManyField):
    """
    Providing a many to many relation that remembers the order of related
    objects.

    Accept a boolean ``sorted`` attribute which specifies if relation is
    ordered or not. Default is set to ``True``. If ``sorted`` is set to
    ``False`` the field will behave exactly like django's ``ManyToManyField``.

    Accept a class ``base_class`` attribute which specifies the base class of
    the intermediate model. It allows to customize the intermediate model.
    """

    def __init__(self, to, sorted=True, base_class=None, **kwargs):  # pylint: disable=redefined-builtin
        self.sorted = sorted
        self.sort_value_field_name = kwargs.pop(
            'sort_value_field_name',
            SORT_VALUE_FIELD_NAME)

        # Base class of through model
        self.base_class = base_class

        super(SortedManyToManyField, self).__init__(to, **kwargs)
        if self.sorted:
            self.help_text = kwargs.get('help_text', None)

    def deconstruct(self):
        # We have to persist custom added options in the ``kwargs``
        # dictionary. For readability only non-default values are stored.
        name, path, args, kwargs = super(SortedManyToManyField, self).deconstruct()
        if self.sort_value_field_name is not SORT_VALUE_FIELD_NAME:
            kwargs['sort_value_field_name'] = self.sort_value_field_name
        if self.sorted is not True:
            kwargs['sorted'] = self.sorted
        return name, path, args, kwargs

    def check(self, **kwargs):
        return (
            super(SortedManyToManyField, self).check(**kwargs) +
            self._check_through_sortedm2m()
        )

    def _check_through_sortedm2m(self):
        rel = get_rel(self)

        # Check if the custom through model of a SortedManyToManyField as a
        # valid '_sort_field_name' attribute
        if self.sorted and rel.through:
            assert hasattr(rel.through, '_sort_field_name'), (
                "The model is used as an intermediate model by "
                "'%s' but has no defined '_sort_field_name' attribute" % rel.through
            )

        return []

    # pylint: disable=inconsistent-return-statements
    def contribute_to_class(self, cls, name, **kwargs):
        if not self.sorted:
            return super(SortedManyToManyField, self).contribute_to_class(cls, name, **kwargs)

        # To support multiple relations to self, it's useful to have a non-None
        # related name on symmetrical relations for internal reasons. The
        # concept doesn't make a lot of sense externally ("you want me to
        # specify *what* on my non-reversible relation?!"), so we set it up
        # automatically. The funky name reduces the chance of an accidental
        # clash.
        rel = get_rel(self)

        if rel.symmetrical and (rel.model == "self" or rel.model == cls._meta.object_name):
            rel.related_name = "%s_rel_+" % name
        elif rel.is_hidden():
            # If the backwards relation is disabled, replace the original
            # related_name with one generated from the m2m field name. Django
            # still uses backwards relations internally and we need to avoid
            # clashes between multiple m2m fields with related_name == '+'.
            rel.related_name = "_%s_%s_+" % (cls.__name__.lower(), name)

        # pylint: disable=bad-super-call
        super(_ManyToManyField, self).contribute_to_class(cls, name, **kwargs)

        # The intermediate m2m model is not auto created if:
        #  1) There is a manually specified intermediate, or
        #  2) The class owning the m2m field is abstract.
        #  3) The class owning the m2m field has been swapped out.
        if not cls._meta.abstract:
            if rel.through:
                def resolve_through_model(_, model):
                    rel.through = model
                lazy_related_operation(resolve_through_model, cls, rel.through)
            elif not cls._meta.swapped:
                rel.through = self.create_intermediate_model(cls)

        # Add the descriptor for the m2m relation
        setattr(cls, self.name, SortedManyToManyDescriptor(self))

        # Set up the accessor for the m2m table name for the relation
        self.m2m_db_table = partial(self._get_m2m_db_table, cls._meta)  # pylint: disable=attribute-defined-outside-init

    def get_internal_type(self):
        return 'ManyToManyField'

    def formfield(self, **kwargs):  # pylint: disable=arguments-differ
        defaults = {}
        if self.sorted:
            defaults['form_class'] = SortedMultipleChoiceField
        defaults.update(kwargs)
        return super(SortedManyToManyField, self).formfield(**defaults)

    def create_intermediate_model(self, klass):
        base_classes = (self.base_class, models.Model) if self.base_class else (models.Model,)

        return create_sortable_many_to_many_intermediary_model(
            self, klass, self.sort_value_field_name,
            base_classes=base_classes)


def create_sortable_many_to_many_intermediary_model(field, klass, sort_field_name, base_classes=None):
    def set_managed(model, related, through):
        through._meta.managed = model._meta.managed or related._meta.managed

    to_model = resolve_relation(klass, field.remote_field.model)
    name = '%s_%s' % (klass._meta.object_name, field.name)
    lazy_related_operation(set_managed, klass, to_model, name)
    base_classes = base_classes if base_classes else (models.Model,)

    # TODO : use autoincrement here ?
    sort_field = models.IntegerField(default=0)

    to = make_model_tuple(to_model)[1]
    from_ = klass._meta.model_name
    if to == from_:
        to = 'to_%s' % to
        from_ = 'from_%s' % from_

    meta = type('Meta', (), {
        'db_table': field._get_m2m_db_table(klass._meta),  # pylint: disable=protected-access
        'auto_created': klass,
        'app_label': klass._meta.app_label,
        'db_tablespace': klass._meta.db_tablespace,
        'unique_together': (from_, to),
        'ordering': (sort_field_name,),
        'verbose_name': _('%(from)s-%(to)s relationship') % {'from': from_, 'to': to},
        'verbose_name_plural': _('%(from)s-%(to)s relationships') % {'from': from_, 'to': to},
        'apps': field.model._meta.apps,
    })

    # Construct and return the new class.
    return type(force_str(name), base_classes, {
        'Meta': meta,
        '__module__': klass.__module__,
        from_: models.ForeignKey(
            klass,
            related_name='%s+' % name,
            db_tablespace=field.db_tablespace,
            db_constraint=field.remote_field.db_constraint,
            on_delete=models.CASCADE,
        ),
        to: models.ForeignKey(
            to_model,
            related_name='%s+' % name,
            db_tablespace=field.db_tablespace,
            db_constraint=field.remote_field.db_constraint,
            on_delete=models.CASCADE,
        ),
        # Sort fields
        sort_field_name: sort_field,
        '_sort_field_name': sort_field_name,
    })



from django.db import models
from django.urls import reverse

from sortedm2m.fields import SortedManyToManyField


class Car(models.Model):
    plate = models.CharField(max_length=50)

    def __str__(self):
        return self.plate


class BaseCarThrough:
    def __str__(self):
        return str(self.car) + " in " + str(self.parkingarea)  # pylint: disable=no-member


class ParkingArea(models.Model):
    name = models.CharField(max_length=50)
    cars = SortedManyToManyField(Car, base_class=BaseCarThrough)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('parkingarea', (self.pk,))



# Django settings for testsite project.
import os

PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))

DEBUG = True

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(PROJECT_ROOT, 'db.sqlite'),
    },
}

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'America/Chicago'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT = os.path.join(PROJECT_ROOT, 'media')
STATIC_ROOT = os.path.join(PROJECT_ROOT, 'static')

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = '/media/'
STATIC_URL = '/static/'

# Make this unique, and don't share it with anybody.
SECRET_KEY = 'define in local settings file'

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'example.urls'

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.admin',
    'django.contrib.staticfiles',

    'sortedm2m',
    'sortedm2m_tests',
    'sortedm2m_tests.migrations_tests',
    'sortedm2m_tests.altersortedmanytomanyfield_tests',

    'example.testapp',
)

TEST_RUNNER = 'django.test.runner.DiscoverRunner'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(PROJECT_ROOT, 'templates'),
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]



"""See ./migrations/ directory for history of these models.

State after 0001_initial:

    class M2MtoSortedM2M(models.Model):
        m2m = models.ManyToManyField('Target')

    class SortedM2MToM2M(models.Model):
        m2m = SortedManyToManyField('Target')

State after 0002_alter_m2m_fields.py

    class M2MToSortedM2M(models.Model):
        m2m = SortedManyToManyField('Target')

    class SortedM2MToM2M(models.Model):
        m2m = models.ManyToManyField('Target')
"""

from django.db import models

from sortedm2m.fields import SortedManyToManyField


class Target(models.Model):
    pass


class M2MToSortedM2M(models.Model):
    """Test model for migration ManyToManyField -> SortedManyToManyField."""

    m2m = SortedManyToManyField('Target')


class SortedM2MToM2M(models.Model):
    """Test model for migration SortedManyToManyField -> ManyToManyField."""

    m2m = models.ManyToManyField('Target')



from unittest.case import skipIf

import django
from django.core.exceptions import FieldDoesNotExist
from django.db import connection
from django.test import TestCase
from django.test.utils import override_settings
from django.utils.encoding import force_str

from sortedm2m.compat import get_field, get_rel
from sortedm2m.fields import SORT_VALUE_FIELD_NAME

from .compat import m2m_set
from .models import Book, DoItYourselfShelf, SelfReference, Shelf, Store, TaggedDoItYourselfShelf


class TestSortedManyToManyField(TestCase):
    model = Shelf

    def setUp(self):
        self.books = [Book.objects.create(name=c) for c in 'abcdefghik']

    def test_adding_items(self):
        shelf = self.model.objects.create()
        self.assertEqual(list(shelf.books.all()), [])

        shelf.books.add(self.books[2])
        self.assertEqual(list(shelf.books.all()), [self.books[2]])

        # adding many with each in one call
        shelf.books.add(self.books[5])
        shelf.books.add(self.books[1])
        self.assertEqual(list(shelf.books.all()), [
            self.books[2],
            self.books[5],
            self.books[1]])

        # adding the same item again won't append another one, order remains
        # the same
        shelf.books.add(self.books[2])
        self.assertEqual(list(shelf.books.all()), [
            self.books[2],
            self.books[5],
            self.books[1]])

        shelf.books.clear()
        self.assertEqual(list(shelf.books.all()), [])

        # adding many with all in the same call
        shelf.books.add(self.books[3], self.books[1], self.books[2])
        self.assertEqual(list(shelf.books.all()), [
            self.books[3],
            self.books[1],
            self.books[2]])

    def test_adding_items_by_pk(self):
        shelf = self.model.objects.create()
        self.assertEqual(list(shelf.books.all()), [])

        shelf.books.add(self.books[2].pk)
        self.assertEqual(list(shelf.books.all()), [self.books[2]])

        shelf.books.add(self.books[5].pk, force_str(self.books[1].pk))
        self.assertEqual(list(shelf.books.all()), [
            self.books[2],
            self.books[5],
            self.books[1]])

        shelf.books.clear()
        self.assertEqual(list(shelf.books.all()), [])

        shelf.books.add(self.books[3].pk, self.books[1], force_str(self.books[2].pk))
        self.assertEqual(list(shelf.books.all()), [
            self.books[3],
            self.books[1],
            self.books[2]])

    def test_set_items(self):
        shelf = self.model.objects.create()
        self.assertEqual(list(shelf.books.all()), [])

        books = self.books[5:2:-1]
        m2m_set(shelf, "books", books)
        self.assertEqual(list(shelf.books.all()), books)

        books.reverse()
        m2m_set(shelf, "books", books)
        self.assertEqual(list(shelf.books.all()), books)

        shelf.books.add(self.books[8])
        self.assertEqual(list(shelf.books.all()), books + [self.books[8]])

        m2m_set(shelf, "books", [])
        self.assertEqual(list(shelf.books.all()), [])

        m2m_set(shelf, "books", [self.books[9]])
        self.assertEqual(list(shelf.books.all()), [
            self.books[9]])

        m2m_set(shelf, "books", [])
        self.assertEqual(list(shelf.books.all()), [])

    def test_set_items_by_pk(self):
        shelf = self.model.objects.create()
        self.assertEqual(list(shelf.books.all()), [])

        books = self.books[5:2:-1]
        m2m_set(shelf, "books", [b.pk for b in books])
        self.assertEqual(list(shelf.books.all()), books)

        m2m_set(shelf, "books", [self.books[5].pk, self.books[2]])
        self.assertEqual(list(shelf.books.all()), [
            self.books[5],
            self.books[2]])

        m2m_set(shelf, "books", [force_str(self.books[8].pk)])
        self.assertEqual(list(shelf.books.all()), [self.books[8]])

    def test_remove_items(self):
        shelf = self.model.objects.create()
        m2m_set(shelf, "books", self.books[2:5])
        self.assertEqual(list(shelf.books.all()), [
            self.books[2],
            self.books[3],
            self.books[4]])

        shelf.books.remove(self.books[3])
        self.assertEqual(list(shelf.books.all()), [
            self.books[2],
            self.books[4]])

        shelf.books.remove(self.books[2], self.books[4])
        self.assertEqual(list(shelf.books.all()), [])

    def test_remove_items_by_pk(self):
        shelf = self.model.objects.create()
        m2m_set(shelf, "books", self.books[2:5])
        self.assertEqual(list(shelf.books.all()), [
            self.books[2],
            self.books[3],
            self.books[4]])

        shelf.books.remove(self.books[3].pk)
        self.assertEqual(list(shelf.books.all()), [
            self.books[2],
            self.books[4]])

        shelf.books.remove(self.books[2], force_str(self.books[4].pk))
        self.assertEqual(list(shelf.books.all()), [])

#    def test_add_relation_by_hand(self):
#        shelf = self.model.objects.create()
#        shelf.books = self.books[2:5]
#        self.assertEqual(list(shelf.books.all()), [
#            self.books[2],
#            self.books[3],
#            self.books[4]])
#
#        shelf.books.create()
#        self.assertEqual(list(shelf.books.all()), [
#            self.books[2],
#            self.books[3],
#            self.books[4]])

    # to enable population of connection.queries
    @override_settings(DEBUG=True)
    def test_prefetch_related_queries_num(self):
        shelf = self.model.objects.create()
        shelf.books.add(self.books[0])

        shelf = self.model.objects.filter(pk=shelf.pk).prefetch_related('books')[0]
        queries_num = len(connection.queries)
        self.assertEqual(queries_num, len(connection.queries))

    def test_prefetch_related_sorting(self):
        shelf = self.model.objects.create()
        books = [self.books[0], self.books[2], self.books[1]]
        m2m_set(shelf, "books", books)

        shelf = self.model.objects.filter(pk=shelf.pk).prefetch_related('books')[0]

        def get_ids(queryset):
            return [obj.id for obj in queryset]

        self.assertEqual(get_ids(shelf.books.all()), get_ids(books))


class TestStringReference(TestSortedManyToManyField):
    """
    Test the same things as ``TestSortedManyToManyField`` but using a model
    that using a string to reference the relation where the m2m field should
    point to.
    """
    model = Store


class TestCustomBaseClass(TestSortedManyToManyField):
    model = DoItYourselfShelf

    def test_base_class_str(self):
        shelf = self.model.objects.create()
        shelf.books.add(self.books[0])
        through_model = shelf.books.through
        instance = through_model.objects.all()[0]
        self.assertEqual(str(instance), "Relationship to {0}".format(instance.book.name))

    def test_custom_sort_value_field_name(self):
        self.assertEqual(len(self.model._meta.many_to_many), 1)
        sortedm2m = self.model._meta.many_to_many[0]
        intermediate_model = get_rel(sortedm2m).through

        # make sure that standard sort field is not used
        self.assertRaises(
            FieldDoesNotExist,
            get_field,
            intermediate_model, SORT_VALUE_FIELD_NAME)

        field = get_field(intermediate_model, 'diy_sort_number')
        self.assertTrue(field)


@skipIf(django.VERSION < (2, 2), 'RelatedManager._add_items() has new through_defaults argument in Django >= 2.2')
class TestCustomThroughClass(TestSortedManyToManyField):
    model = TaggedDoItYourselfShelf

    def test_base_class_str(self):
        shelf = self.model.objects.create()
        shelf.books.add(self.books[0])
        through_model = shelf.books.through
        instance = through_model.objects.all()[0]
        self.assertEqual(str(instance), "Relationship to {0} tagged as <>".format(instance.book.name))

    def test_through_defaults(self):
        shelf = self.model.objects.create()
        shelf.books.add(*self.books[3:5], through_defaults={'tags': 'A'})
        shelf.books.add(self.books[2])
        shelf.books.add(*self.books[:2], through_defaults={'tags': 'B'})

        # ordering is kept
        self.assertEqual(list(shelf.books.all()),
                         self.books[3:5] + [self.books[2]] + self.books[:2])

        through_model = shelf.books.through
        rels = [(o.book_id, o.tags_diy_sort_number, o.tags) for o in through_model.objects.all()]

        self.assertEqual(rels, [
            (self.books[3].pk, 1, 'A'),
            (self.books[4].pk, 2, 'A'),
            (self.books[2].pk, 3, ''),
            (self.books[0].pk, 4, 'B'),
            (self.books[1].pk, 5, 'B'),
        ])


class TestSelfReference(TestCase):
    def test_self_adding(self):
        s1 = SelfReference.objects.create()
        s2 = SelfReference.objects.create()
        s3 = SelfReference.objects.create()
        s4 = SelfReference.objects.create()
        s1.me.add(s3)
        s1.me.add(s4, s2)

        self.assertEqual(list(s1.me.all()), [s3, s4, s2])



from django.db import models

from sortedm2m.fields import SortedManyToManyField


class BaseBookThrough(object):  # pylint: disable=useless-object-inheritance

    def __str__(self):
        return "Relationship to {0}".format(self.book.name)  # pylint: disable=no-member


class Shelf(models.Model):
    books = SortedManyToManyField('Book', related_name='shelves', base_class=BaseBookThrough)


class Book(models.Model):
    name = models.CharField(max_length=50)

    def __str__(self):
        return self.name


class DoItYourselfShelf(models.Model):
    books = SortedManyToManyField(
        Book,
        sort_value_field_name='diy_sort_number',
        related_name='diy_shelves',
        base_class=BaseBookThrough
    )


class TaggedDoItYourselfShelf(models.Model):
    books = SortedManyToManyField(
        Book,
        sort_value_field_name='diy_sort_number',
        related_name='tagged_diy_shelves',
        through='TagThrough',
        through_fields=('shelf', 'book')
    )


class TagThrough(models.Model):
    _sort_field_name = 'tags_diy_sort_number'

    book = models.ForeignKey(Book, on_delete=models.CASCADE)
    shelf = models.ForeignKey(TaggedDoItYourselfShelf, on_delete=models.CASCADE)
    tags = models.CharField(max_length=50, blank=True, default='')
    tags_diy_sort_number = models.IntegerField(default=0)

    def __str__(self):
        return "Relationship to {0} tagged as <{1}>".format(self.book.name, self.tags)


class Store(models.Model):
    books = SortedManyToManyField('sortedm2m_tests.Book', related_name='stores')


class MessyStore(models.Model):
    books = SortedManyToManyField(
        'Book',
        sorted=False,
        related_name='messy_stores'
    )


class SelfReference(models.Model):
    me = SortedManyToManyField('self', related_name='hide+')

    def __str__(self):
        return str(self.pk)



from test_project.settings import *  # pylint: disable=unused-wildcard-import,wildcard-import

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'HOST': 'localhost',
        'NAME': 'sortedm2m',
        "USER": os.environ.get('DJANGO_DB_USER', 'sortedm2m'),
        'PASSWORD': os.environ.get('DJANGO_DB_PASSWORD', 'sortedm2m'),
    },
}



import contextlib
import sys
from io import StringIO


@contextlib.contextmanager
def capture_stdout(target=None):
    original = sys.stdout
    if target is None:
        target = StringIO()
    sys.stdout = target
    yield target
    target.seek(0)
    sys.stdout = original


@contextlib.contextmanager
def capture_stderr(target=None):
    original = sys.stderr
    if target is None:
        target = StringIO()
    sys.stderr = target
    yield target
    target.seek(0)
    sys.stderr = original



from django.contrib import admin

from example.testapp.models import Car, ParkingArea


class ParkingAreaAdmin(admin.ModelAdmin):
    fieldsets = (
        ('bla', {
            'classes': ('wide',),
            'fields': (
                'name',
                'cars',
            ),
        }),
    )


admin.site.register(Car)
admin.site.register(ParkingArea, ParkingAreaAdmin)



from django import forms
from django.test import TestCase
from django.utils.encoding import force_str

from sortedm2m.forms import SortedMultipleChoiceField

from .models import Book, MessyStore, Shelf


class SortedForm(forms.Form):
    values = SortedMultipleChoiceField(
        queryset=Book.objects.all(),
        required=False)


class SortedNameForm(forms.Form):
    values = SortedMultipleChoiceField(
        queryset=Book.objects.all(),
        to_field_name='name')


class TestSortedFormField(TestCase):  # pylint: disable=too-many-instance-attributes
    def setUp(self):
        self.books = [Book.objects.create(name=c) for c in 'abcdefghik']
        (self.a, self.b, self.c, self.d, self.e, self.f, self.g, self.h, self.i, self.k,) = self.books

    def test_empty_field(self):
        form = SortedForm({'values': []})
        self.assertTrue(form.is_valid())
        self.assertFalse(form.cleaned_data['values'])

    def test_sorted_field_input(self):
        form = SortedForm({'values': [self.d.pk, self.b.pk, self.i.pk]})
        self.assertTrue(form.is_valid())
        self.assertEqual(
            list(form.cleaned_data['values']),
            [
                self.books[3],
                self.books[1],
                self.books[8],
            ]
        )

        form = SortedForm({'values': [book.pk for book in self.books[::-1]]})
        self.assertTrue(form.is_valid())
        self.assertEqual(list(form.cleaned_data['values']), self.books[::-1])

    def test_sorted_field_input_with_field_name(self):
        form = SortedNameForm({'values': ['d', 'b', 'i']})
        self.assertTrue(form.is_valid())
        self.assertEqual(
            list(form.cleaned_data['values']),
            [
                self.books[3],
                self.books[1],
                self.books[8],
            ]
        )

        form = SortedNameForm({'values': [book.name for book in self.books[::-1]]})
        self.assertTrue(form.is_valid())
        self.assertEqual(list(form.cleaned_data['values']), self.books[::-1])

    def test_form_field_on_model_field(self):
        class ShelfForm(forms.ModelForm):
            class Meta:
                model = Shelf
                fields = ('books',)

        form = ShelfForm()
        self.assertTrue(isinstance(form.fields['books'], SortedMultipleChoiceField))

        class MessyStoreForm(forms.ModelForm):
            class Meta:
                model = MessyStore
                fields = ('books',)

        form = MessyStoreForm()
        self.assertFalse(isinstance(form.fields['books'], SortedMultipleChoiceField))
        self.assertTrue(isinstance(form.fields['books'], forms.ModelMultipleChoiceField))

    # regression test
    def test_form_field_with_only_one_value(self):
        form = SortedForm({'values': ''})
        self.assertEqual(len(form.errors), 0)
        form = SortedForm({'values': str(self.a.pk)})
        self.assertEqual(len(form.errors), 0)
        form = SortedForm({'values': '{a.pk},{b.pk}'.format(a=self.a, b=self.b)})
        self.assertEqual(len(form.errors), 0)

    def test_for_attribute_in_label(self):
        form = SortedForm()
        rendered = force_str(form['values'])
        self.assertTrue(' for="id_values_0"' in rendered)

        form = SortedForm(prefix='prefix')
        rendered = force_str(form['values'])
        self.assertTrue(' for="id_prefix-values_0"' in rendered)

        # check that it will be escaped properly

        form = SortedForm(prefix='hacking"><a href="TRAP">')
        rendered = force_str(form['values'])
        self.assertTrue(' for="id_hacking&quot;&gt;&lt;a href=&quot;TRAP&quot;&gt;-values_0"' in rendered)

    def test_input_id_is_escaped(self):
        form = SortedForm(prefix='hacking"><a href="TRAP">')
        rendered = force_str(form['values'])
        self.assertTrue(' id="id_hacking&quot;&gt;&lt;a href=&quot;TRAP&quot;&gt;-values_0"' in rendered)

    def test_form_field_detects_reordering(self):
        form = SortedForm({'values': '{a.pk},{b.pk}'.format(a=self.a, b=self.b)})
        form.fields['values'].initial = [self.b.pk, self.a.pk]
        self.assertTrue('values' in form.changed_data)



import django.views.static
from django.conf import settings
from django.conf.urls import include, url
from django.contrib import admin
from django.http import HttpResponse

import example.testapp.views

admin.autodiscover()


def handle404(request, exception):
    return HttpResponse('404')


def handle500(request):
    return HttpResponse('500')


handler404 = 'example.urls.handle404'
handler500 = 'example.urls.handle500'


if django.VERSION < (1, 9):
    urlpatterns = [url(r'^admin/', include(admin.site.urls), name="admin")]
else:
    urlpatterns = [url(r'^admin/', admin.site.urls, name="admin")]

urlpatterns += [
    url(r'^media/(.*)$', django.views.static.serve, {'document_root': settings.MEDIA_ROOT}),
    url(r'^parkingarea/(?P<pk>\d+)/$', example.testapp.views.parkingarea_update, name='parkingarea'),
    url(r'^', include('django.contrib.staticfiles.urls')),
]



#!/usr/bin/env python
import codecs
import os
import re

from setuptools import setup


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    matches = re.search(
        r"__version__[\s]+=[\s]+['\"](?P<version>[^'\"]+)['\"]",
        open(os.path.join(package, '__init__.py')).read(),
        re.M
    )

    return matches.group(1) if matches else None


def read(filename):
    return codecs.open(os.path.join(os.path.dirname(__file__), filename),
                       encoding='utf8').read()


long_description = '\n\n'.join((
    read('README.rst'),
    read('CHANGES.rst'),
))

setup(
    name='django-sortedm2m',
    version=get_version('sortedm2m'),
    url='http://github.com/jazzband/django-sortedm2m',
    license='BSD',
    description="Drop-in replacement for django's many to many field with sorted relations.",
    long_description=long_description,
    author=u'Gregor Müllegger',
    author_email='gregor@muellegger.de',
    packages=['sortedm2m'],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



from django.db import models
from django.db.migrations.operations import AlterField

from .compat import get_apps_from_state, get_field, get_rel


class AlterSortedManyToManyField(AlterField):
    """A migration operation to transform a ManyToManyField into a
    SortedManyToManyField and vice versa."""

    def database_forwards(self, app_label, schema_editor, from_state, to_state):
        to_apps = get_apps_from_state(to_state)
        to_model = to_apps.get_model(app_label, self.model_name)
        if self.allow_migrate_model(schema_editor.connection.alias, to_model):
            to_field = get_field(to_model, self.name)

            from_apps = get_apps_from_state(from_state)
            from_model = from_apps.get_model(app_label, self.model_name)
            from_field = get_field(from_model, self.name)

            to_m2m_model = get_rel(to_field).through
            from_m2m_model = get_rel(from_field).through

            # M2M -> SortedM2M
            if getattr(to_field, 'sorted', False):
                self.add_sort_value_field(schema_editor, to_m2m_model)
            # SortedM2M -> M2M
            elif getattr(from_field, 'sorted', False):
                self.remove_sort_value_field(schema_editor, from_m2m_model)
            else:
                raise TypeError(
                    '{operation} should only be used when changing a '
                    'SortedManyToManyField into a ManyToManyField or a '
                    'ManyToManyField into a SortedManyToManyField.'
                    .format(operation=self.__class__.__name__))

    def database_backwards(self, app_label, schema_editor, from_state, to_state):
        from_apps = get_apps_from_state(from_state)
        from_model = from_apps.get_model(app_label, self.model_name)
        from_field = get_field(from_model, self.name)

        to_apps = get_apps_from_state(to_state)
        to_model = to_apps.get_model(app_label, self.model_name)

        if self.allow_migrate_model(schema_editor.connection.alias, to_model):
            to_field = get_field(to_model, self.name)
            from_m2m_model = get_rel(from_field).through
            to_m2m_model = get_rel(to_field).through

            # The `to_state` is the OLDER state.

            # M2M -> SortedM2M (backwards)
            if getattr(to_field, 'sorted', False):
                self.add_sort_value_field(schema_editor, to_m2m_model)
            # SortedM2M -> M2M (backwards)
            elif getattr(from_field, 'sorted', False):
                self.remove_sort_value_field(schema_editor, from_m2m_model)
            else:
                raise TypeError(
                    '{operation} should only be used when changing a '
                    'SortedManyToManyField into a ManyToManyField or a '
                    'ManyToManyField into a SortedManyToManyField.'
                    .format(operation=self.__class__.__name__))

    def add_sort_value_field(self, schema_editor, model):
        field = self.make_sort_by_field(model)
        schema_editor.add_field(model, field)

    @staticmethod
    def remove_sort_value_field(schema_editor, model):
        field = get_field(model, model._sort_field_name)  # pylint: disable=protected-access
        schema_editor.remove_field(model, field)

    @staticmethod
    def make_sort_by_field(model):
        field_name = model._sort_field_name  # pylint: disable=protected-access
        field = models.IntegerField(name=field_name, default=0)
        field.set_attributes_from_name(field_name)
        return field



from itertools import chain

from django import forms
from django.template.loader import render_to_string
from django.utils.encoding import force_str
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe


class SortedCheckboxSelectMultiple(forms.CheckboxSelectMultiple):
    class Media:
        js = (
            'sortedm2m/widget.js',
            'sortedm2m/jquery-ui.js',
        )
        css = {'screen': (
            'sortedm2m/widget.css',
        )}

    def build_attrs(self, attrs=None, **kwargs):  # pylint: disable=arguments-differ
        attrs = dict(attrs or {}, **kwargs)
        attrs = super(SortedCheckboxSelectMultiple, self).build_attrs(attrs)
        classes = attrs.setdefault('class', '').split()
        classes.append('sortedm2m')
        attrs['class'] = ' '.join(classes)
        return attrs

    def render(self, name, value, attrs=None, choices=(), renderer=None):  # pylint: disable=arguments-differ
        if value is None:
            value = []
        has_id = attrs and 'id' in attrs
        final_attrs = self.build_attrs(attrs, name=name)

        # Normalize to strings
        str_values = [force_str(v) for v in value]

        selected = []
        unselected = []

        for i, (option_value, option_label) in enumerate(chain(self.choices, choices)):
            # If an ID attribute was given, add a numeric index as a suffix,
            # so that the checkboxes don't all have the same ID attribute.
            if has_id:
                final_attrs = dict(final_attrs, id='%s_%s' % (attrs['id'], i))
                label_for = ' for="%s"' % conditional_escape(final_attrs['id'])
            else:
                label_for = ''

            cb = forms.CheckboxInput(final_attrs, check_test=lambda value: value in str_values)
            option_value = force_str(option_value)
            rendered_cb = cb.render(name, option_value)
            option_label = conditional_escape(force_str(option_label))
            item = {
                'label_for': label_for,
                'rendered_cb': rendered_cb,
                'option_label': option_label,
                'option_value': option_value
            }
            if option_value in str_values:
                selected.append(item)
            else:
                unselected.append(item)

        # Reorder `selected` array according str_values which is a set of `option_value`s in the
        # order they should be shown on screen
        ordered = []
        for s in str_values:
            for select in selected:
                if s == select['option_value']:
                    ordered.append(select)
        selected = ordered

        html = render_to_string(
            'sortedm2m/sorted_checkbox_select_multiple_widget.html',
            {'selected': selected, 'unselected': unselected})
        return mark_safe(html)

    def value_from_datadict(self, data, files, name):
        value = data.get(name, None)
        if isinstance(value, (str,)):
            return [v for v in value.split(',') if v]
        return value


class SortedMultipleChoiceField(forms.ModelMultipleChoiceField):
    widget = SortedCheckboxSelectMultiple

    def clean(self, value):
        queryset = super(SortedMultipleChoiceField, self).clean(value)
        if value is None or not hasattr(queryset, '__iter__'):
            return queryset
        key = self.to_field_name or 'pk'
        objects = dict((force_str(getattr(o, key)), o) for o in queryset)
        return [objects[force_str(val)] for val in value]

    def has_changed(self, initial, data):
        if initial is None:
            initial = []
        if data is None:
            data = []
        if len(initial) != len(data):
            return True
        initial_set = [force_str(value) for value in self.prepare_value(initial)]
        data_set = [force_str(value) for value in data]
        return data_set != initial_set
